@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -30,60 +32,145 @@ Base<elem_type,derived>::get_ref() const
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 Base<elem_type,derived>::print(const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const quasi_unwrap<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_print(extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::print(get_cout_stream(), tmp.M, true);
   }
 
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 Base<elem_type,derived>::print(std::ostream& user_stream, const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const quasi_unwrap<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_print(user_stream, extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::print(user_stream, tmp.M, true);
   }
   
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 Base<elem_type,derived>::raw_print(const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const quasi_unwrap<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_raw_print(extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::print(get_cout_stream(), tmp.M, false);
   }
 
 
 
 template<typename elem_type, typename derived>
-arma_cold
 inline
 void
 Base<elem_type,derived>::raw_print(std::ostream& user_stream, const std::string extra_text) const
   {
+  arma_extra_debug_sigprint();
+  
   const quasi_unwrap<derived> tmp( (*this).get_ref() );
   
-  tmp.M.impl_raw_print(user_stream, extra_text);
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::print(user_stream, tmp.M, false);
+  }
+
+
+
+template<typename elem_type, typename derived>
+inline
+void
+Base<elem_type,derived>::brief_print(const std::string extra_text) const
+  {
+  arma_extra_debug_sigprint();
+  
+  const quasi_unwrap<derived> tmp( (*this).get_ref() );
+  
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = get_cout_stream().width();
+    
+    get_cout_stream() << extra_text << '\n';
+    
+    get_cout_stream().width(orig_width);
+    }
+  
+  arma_ostream::brief_print(get_cout_stream(), tmp.M);
+  }
+
+
+
+template<typename elem_type, typename derived>
+inline
+void
+Base<elem_type,derived>::brief_print(std::ostream& user_stream, const std::string extra_text) const
+  {
+  arma_extra_debug_sigprint();
+  
+  const quasi_unwrap<derived> tmp( (*this).get_ref() );
+  
+  if(extra_text.length() != 0)
+    {
+    const std::streamsize orig_width = user_stream.width();
+    
+    user_stream << extra_text << '\n';
+    
+    user_stream.width(orig_width);
+    }
+  
+  arma_ostream::brief_print(user_stream, tmp.M);
   }
 
 
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 elem_type
 Base<elem_type,derived>::min() const
   {
@@ -94,7 +181,6 @@ Base<elem_type,derived>::min() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 elem_type
 Base<elem_type,derived>::max() const
   {
@@ -171,7 +257,6 @@ Base<elem_type,derived>::max(uword& row_of_max_val, uword& col_of_max_val) const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 uword
 Base<elem_type,derived>::index_min() const
   {
@@ -195,7 +280,6 @@ Base<elem_type,derived>::index_min() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 uword
 Base<elem_type,derived>::index_max() const
   {
@@ -219,7 +303,6 @@ Base<elem_type,derived>::index_max() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_symmetric() const
   {
@@ -260,7 +343,6 @@ Base<elem_type,derived>::is_symmetric() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_symmetric(const typename get_pod_type<elem_type>::result tol) const
   {
@@ -292,7 +374,6 @@ Base<elem_type,derived>::is_symmetric(const typename get_pod_type<elem_type>::re
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_hermitian() const
   {
@@ -345,7 +426,6 @@ Base<elem_type,derived>::is_hermitian() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_hermitian(const typename get_pod_type<elem_type>::result tol) const
   {
@@ -377,7 +457,6 @@ Base<elem_type,derived>::is_hermitian(const typename get_pod_type<elem_type>::re
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_zero(const typename get_pod_type<elem_type>::result tol) const
   {
@@ -411,15 +490,15 @@ Base<elem_type,derived>::is_zero(const typename get_pod_type<elem_type>::result
       const T val_real = access::tmp_real(val);
       const T val_imag = access::tmp_imag(val);
       
-      if(std::abs(val_real) > tol)  { return false; }
-      if(std::abs(val_imag) > tol)  { return false; }
+      if(eop_aux::arma_abs(val_real) > tol)  { return false; }
+      if(eop_aux::arma_abs(val_imag) > tol)  { return false; }
       }
     }
   else  // not complex
     {
     for(uword i=0; i<n_elem; ++i)
       {
-      if(std::abs(Pea[i]) > tol)  { return false; }
+      if(eop_aux::arma_abs(Pea[i]) > tol)  { return false; }
       }
     }
   
@@ -430,7 +509,6 @@ Base<elem_type,derived>::is_zero(const typename get_pod_type<elem_type>::result
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_trimatu() const
   {
@@ -449,7 +527,6 @@ Base<elem_type,derived>::is_trimatu() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_trimatl() const
   {
@@ -468,7 +545,6 @@ Base<elem_type,derived>::is_trimatl() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_diagmat() const
   {
@@ -508,7 +584,6 @@ Base<elem_type,derived>::is_diagmat() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_empty() const
   {
@@ -523,7 +598,6 @@ Base<elem_type,derived>::is_empty() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_square() const
   {
@@ -538,7 +612,6 @@ Base<elem_type,derived>::is_square() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_vec() const
   {
@@ -555,7 +628,6 @@ Base<elem_type,derived>::is_vec() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_colvec() const
   {
@@ -572,7 +644,6 @@ Base<elem_type,derived>::is_colvec() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_rowvec() const
   {
@@ -589,7 +660,6 @@ Base<elem_type,derived>::is_rowvec() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::is_finite() const
   {
@@ -634,7 +704,6 @@ Base<elem_type,derived>::is_finite() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::has_inf() const
   {
@@ -679,7 +748,6 @@ Base<elem_type,derived>::has_inf() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base<elem_type,derived>::has_nan() const
   {
@@ -723,7 +791,7 @@ Base<elem_type,derived>::has_nan() const
 
 
 template<typename elem_type, typename derived>
-arma_inline
+inline
 const Op<derived,op_vectorise_col>
 Base<elem_type, derived>::as_col() const
   {
@@ -733,7 +801,7 @@ Base<elem_type, derived>::as_col() const
 
 
 template<typename elem_type, typename derived>
-arma_inline
+inline
 const Op<derived,op_vectorise_row>
 Base<elem_type, derived>::as_row() const
   {
@@ -746,18 +814,17 @@ Base<elem_type, derived>::as_row() const
 // extra functions defined in Base_extra_yes
 
 template<typename elem_type, typename derived>
-arma_inline
-const Op<derived,op_inv>
+inline
+const Op<derived,op_inv_gen_default>
 Base_extra_yes<elem_type, derived>::i() const
   {
-  return Op<derived,op_inv>(static_cast<const derived&>(*this));
+  return Op<derived,op_inv_gen_default>(static_cast<const derived&>(*this));
   }
 
 
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base_extra_yes<elem_type,derived>::is_sympd() const
   {
@@ -783,7 +850,6 @@ Base_extra_yes<elem_type,derived>::is_sympd() const
 
 template<typename elem_type, typename derived>
 inline
-arma_warn_unused
 bool
 Base_extra_yes<elem_type,derived>::is_sympd(typename get_pod_type<elem_type>::result tol) const
   {
@@ -825,7 +891,7 @@ Base_eval_Mat<elem_type, derived>::eval() const
 // extra functions defined in Base_eval_expr
 
 template<typename elem_type, typename derived>
-arma_inline
+inline
 Mat<elem_type>
 Base_eval_expr<elem_type, derived>::eval() const
   {
