@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -27,7 +29,17 @@ class op_diagmat
   template<typename T1>
   inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_diagmat>& X);
   
-  // TODO: implement specialised handling of Op<Glue<T1,T2,glue_times>,op_diagmat>
+  template<typename T1>
+  inline static void apply(Mat<typename T1::elem_type>& out, const Proxy<T1>& P);
+  
+  template<typename T1, typename T2>
+  inline static void apply(Mat<typename T1::elem_type>& out, const Op< Glue<T1,T2,glue_times>, op_diagmat>& X);
+  
+  template<typename T1, typename T2>
+  inline static void apply_times(Mat<typename T1::elem_type>& out, const T1& X, const T2& Y, const typename arma_not_cx<typename T1::elem_type>::result* junk = nullptr);
+  
+  template<typename T1, typename T2>
+  inline static void apply_times(Mat<typename T1::elem_type>& out, const T1& X, const T2& Y, const typename arma_cx_only<typename T1::elem_type>::result* junk = nullptr);
   };
 
 
@@ -38,10 +50,10 @@ class op_diagmat2
   public:
   
   template<typename T1>
-  inline static void apply(Mat<typename T1::elem_type>& out, const Proxy<T1>& P, const uword row_offset, const uword col_offset);
+  inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_diagmat2>& X);
   
   template<typename T1>
-  inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_diagmat2>& X);
+  inline static void apply(Mat<typename T1::elem_type>& out, const Proxy<T1>& P, const uword row_offset, const uword col_offset);
   };
 
 
